// DiskEditor 
// ==========
// ANT Systems TDirectDisk Component Sample.
// Version 2.00L

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, DirectDisk, ComCtrls, Spin, Mask;

type
  TSectorArray = Array[0..2047] of Byte;

  TForm1 = class(TForm)
    StringGrid1: TStringGrid;
    DirDisk1: TDirDisk;
    bReadSec: TButton;
    seFirstSector: TSpinEdit;
    StatusBar1: TStatusBar;
    bWriteSec: TButton;
    Label1: TLabel;
    Label2: TLabel;
    eDrive: TEdit;
    procedure bReadSecClick(Sender: TObject);
    procedure StringGrid1GetEditText(Sender: TObject; ACol, ARow: LongInt;
      var Value: String);
    procedure StringGrid1KeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure StringGrid1SelectCell(Sender: TObject; Col, Row: LongInt;
      var CanSelect: Boolean);
    procedure bWriteSecClick(Sender: TObject);
    procedure eDriveChange(Sender: TObject);
    procedure eDriveExit(Sender: TObject);
  private
    { Private declarations }
    A,B: TSectorArray;
    OldCol, OldRow: LongInt;
    procedure BytesToGrid;
    procedure OnFocusChange;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.bReadSecClick(Sender: TObject);
var I: Integer;
begin
  with DirDisk1 do begin
    DriveName:=eDrive.Text[1];
    if (TotalNumberOfClusters*SectorsPerCluster)>Cardinal(seFirstSector.Value)
      then begin
      FirstSector:=seFirstSector.Value;
      Sectors:=1;

      //BIOS don't support sector size more then 2K,
      //therefore you don't need the next checking:
      if BytesPerSector<=Cardinal(SizeOf(A)) then begin  //always
        if ReadSectors(A) then begin
          BytesToGrid;
          for i := 0 to SizeOf(A)-1 do B[i]:=A[i];
          StatusBar1.Panels[0].Text:=Format('Sector #%D',[seFirstSector.Value]);
          bWriteSec.Enabled:=True;
        end else
          StatusBar1.Panels[0].Text:='Error Sector Read!';

      end else  StatusBar1.Panels[0].Text:='Too big sector!'; //never
      StringGrid1.Col:=1;
      StringGrid1.Row:=1;
      StatusBar1.Panels[1].Text:='';
    end else begin
      StatusBar1.Panels[0].Text:=Format('Sector #%D',[FirstSector]);
      StatusBar1.Panels[1].Text:=
          Format('Sector %D not found!',[seFirstSector.Value]);
      seFirstSector.Value:=FirstSector;
    end;  (* num sectors on disk check *)
  end;  (* with DirDisk1 *)
end;

procedure TForm1.bWriteSecClick(Sender: TObject);
begin
  if not DirDisk1.WriteSectors(B) then
     StatusBar1.SimpleText:='Error Sector Write!';;
end;

procedure TForm1.BytesToGrid;
var
  I,J,K: Integer;
  C: Char;
begin
  for i:=0 to (DirDisk1.BytesPerSector div 16)-1 do begin  //rows
    StringGrid1.Cells[0,i+1]:=IntToHex(i*16,4)+':';  //offsets
    for j:=1 to 16 do begin  //columns
      K:=A[j-1+16*i];
      StringGrid1.Cells[j,i+1]:=IntToHex(K,2);
      C:=Chr(K);
      if C<' ' then C:='.';
      StringGrid1.Cells[j+17,i+1]:=C;
    end;
  end;
end;

procedure TForm1.StringGrid1GetEditText(Sender: TObject; ACol,
  ARow: LongInt; var Value: String);
begin
  OldCol:=ACol; OldRow:=ARow;
end;

procedure TForm1.StringGrid1KeyPress(Sender: TObject; var Key: Char);
begin
  if Key<>#8 then begin
    Key:=UpCase(Key);
    if not (Key in ['0'..'9','A'..'F']) then Key:=#0;
  end;
  if Key<>#0 then StatusBar1.Panels[1].Text:='Modified!';
end;

procedure TForm1.OnFocusChange;
var
  I: LongInt;
  C: Char;
begin
  I:=OldCol-1+(OldRow-1)*16;
  with StringGrid1 do begin
    if Length(Cells[OldCol,OldRow])>2 then begin
      Cells[OldCol,OldRow]:=IntToHex(A[I],2);
      MessageBeep(0);
    end else
    try
      B[I]:=StrToInt('$'+Cells[OldCol,OldRow]);
      C:=Chr(B[I]);
      if C<' ' then C:='.';
      Cells[OldCol+17,OldRow]:=C;
    except
      Cells[OldCol,OldRow]:=IntToHex(B[I],2);
    end;
  end;    (* with *)
end;

procedure TForm1.StringGrid1SelectCell(Sender: TObject; Col, Row: LongInt;
  var CanSelect: Boolean);
begin
  if Col>16 then CanSelect:=False;
  OnFocusChange;
end;

procedure TForm1.eDriveChange(Sender: TObject);
begin
  if Length(eDrive.Text)>0 then begin
    DirDisk1.DriveName:=eDrive.Text[1];
    if not DirDisk1.TrueDisk then begin
      eDrive.Text:=DirDisk1.DriveName;
      MessageBeep(0);
    end else begin
      bWriteSec.Enabled:=False;
    end;
  end;
end;

procedure TForm1.eDriveExit(Sender: TObject);
begin
  if eDrive.Modified then begin
    eDrive.Text:=DirDisk1.DriveName;
    bReadSecClick(Sender);
    eDrive.Modified:=False;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var I: Integer;
begin
  OldCol:=1;
  OldRow:=1;
  with StringGrid1 do begin
    ColWidths[0]:=50;
    for i := 18 to 33 do begin
      TabStops[i]:=False;
      ColWidths[i]:=11;
    end;    (* for *)
    //grid captions:
    for i := 1 to 16 do Cells[i,0]:=IntToHex(i-1,2);
    for i := 18 to 33 do Cells[i,0]:=IntToHex(i-18,1);
    Cells[0,0]:='Offset';
    Form1.Width:=GridWidth+32;
    bReadSecClick(Sender);
  end;    (* with *)
end;

end.
